/*
 * STM32F4 Wifi flight controller
 * Copyright (C) 2012-2014 Adam Williams <broadcast at earthling dot net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef NET_H
#define NET_H


#include "arm/config.h"
#include "linux.h"
#include <stdint.h>
#include "usbh_conf.h"




#define MAX_STATIONS 8



#define BEACON_INT 100

#ifndef ALWAYS_RX
#define IN_BUFFER_SIZE 2048
#endif

#define OUT_BUFFER_LEAD 128
// size of left eye fifo
#define LEFT_FIFO_SIZE ((PICTURE_FRAGMENT_SIZE + SEGMENT_HEADER_SIZE) * 8)
#define PACKET_OFFSET 56



#define BOOTP_BROADCAST 0x8000

#define DHCP_REQUEST        1
#define DHCP_REPLY          2
#define DHCP_HTYPE_ETHERNET 1
#define DHCP_HLEN_ETHERNET  6
#define DHCP_MSG_LEN      236

#define DHCPC_SERVER_PORT  67
#define DHCPC_CLIENT_PORT  68

#define DHCPDISCOVER  1
#define DHCPOFFER     2
#define DHCPREQUEST   3
#define DHCPDECLINE   4
#define DHCPACK       5
#define DHCPNAK       6
#define DHCPRELEASE   7

#define DHCP_OPTION_SUBNET_MASK   1
#define DHCP_OPTION_ROUTER        3
#define DHCP_OPTION_DNS_SERVER    6
#define DHCP_OPTION_REQ_IPADDR   50
#define DHCP_OPTION_LEASE_TIME   51
#define DHCP_OPTION_MSG_TYPE     53
#define DHCP_OPTION_SERVER_ID    54
#define DHCP_OPTION_REQ_LIST     55
#define DHCP_OPTION_END         255



typedef struct
{
// Room to put headers in without copying data
	unsigned char out_buffer_lead[OUT_BUFFER_LEAD];
	unsigned char out_buffer[ARM_OUT_BUFFER_SIZE];
#ifndef ALWAYS_RX
	unsigned char in_buffer[IN_BUFFER_SIZE];
	int bytes_received;
#endif

#ifdef PIC_USE_MRF49XA
	unsigned char beacon[COPTER_SERIAL_IN_SIZE];
	int beacon_offset;
#endif

	void (*current_function)();
// used in wait_tx to determine the next state
	void (*next_function)();

	int net_initialized;
	struct ieee80211_hw hw;
	struct ieee80211_vif vif;

// last authorized station
	unsigned char authed_mac[ETH_ALEN];
	int authed;

// stations
// index + 1 is the AID
	unsigned char *station_macs[MAX_STATIONS];
	int total_stations;
	int connected;

// IP addresses
	unsigned char self_ip[4];
// Disable the motor
	int settings_changed;

// Port ground station sent packet from
	int src_port;
// Port ground station sent packet to
	int dst_port;

// MAC addresses in last received packet
	unsigned char *src_mac;
	unsigned char *dst_mac;
	unsigned char *filter_mac;

// for packet forwarding, copter uses it to forward beacons to the ground
	unsigned char ground_mac[ETH_ALEN];

// IP addresses in the last received packet
	unsigned char *src_ip;
	unsigned char *dst_ip;

// Station index of last received packet
	int last_station;

	int hc_num_in;
	int hc_num_out[2];
	int hc_num_int;

	int seq_ctrl;

// DNS records
	unsigned char *a_record;
// Length including trailing 0
	int a_record_len;
	unsigned char *ptr_record;
	int ptr_record_len;

// TCP bits
// TODO: must be in unique connection structs
	unsigned int sequence_number;
	unsigned int ack_number;
	unsigned char *tcp_payload;
	int tcp_size;


// Offset in output frame
	int frame_offset;
// Number of the frame
	int frame_number;
// Number of times the current frame offset has been sent
	int resends_done;
// Fragments sent after the last beacon
	int picture_fragments;
// Endpoint of last transmission
	int current_qnum;
// Resend last transmission
	int resends;
// time last beacon was sent
	int last_beacon;
	unsigned char *current_data;
	int current_size;

// left eye fifo
	unsigned char *left_fifo;
	int left_usage;
	int left_write;
	int left_read;

// byte totals
	int total_rx;
	int total_tx;
// packet totals
	int total_mgmt;
	int total_data;
	int total_udp;
	int total_arp;
	int total_icmp;
	int total_tcp;
} net_t;



extern net_t net;



void init_net();
void init_net2();

#define handle_net() net.current_function();

struct ieee80211_hw *ieee80211_alloc_hw(size_t priv_data_len,
					const struct ieee80211_ops *ops);
// get MAC address
unsigned char* get_mac();
void net_start_receive();
// data is net.out_buffer
void send_udp(int size,
	void (*next_function)(),
	int need_resends);

void net_wait_tx();



#endif
