#include "linux.h"
#include "wifi.h"

#include "core.h"
#include "net.h"
#include "rtl_usb.h"
#include "uart.h"
#include "usb.h"

// RTL USB port for Marcy 2

#ifdef USE_WIFI

extern const struct rtl_hal_cfg rtl92cu_hal_cfg;
static int total_tx;


#define	REALTEK_USB_VENQT_READ			0xC0
#define	REALTEK_USB_VENQT_WRITE			0x40
#define REALTEK_USB_VENQT_CMD_REQ		0x05
#define	REALTEK_USB_VENQT_CMD_IDX		0x00
//#define DURATION_VALUE 0x0100
#define DURATION_VALUE 0x013a

// Map queue number to out endpoint number
/*
 * queue=0x0 endpoint=0x3
 * queue=0x1 endpoint=0x3
 * queue=0x2 endpoint=0x2
 * queue=0x3 endpoint=0x2
 * queue=0x4 endpoint=0x2
 * queue=0x5 endpoint=0x2
 * queue=0x6 endpoint=0x2
 */
const int ep_mapping[] = 
{
	1,
	1,
	0,
	0,
	0,
	0,
	0
};




void rtl_usb_receive()
{


#ifdef ALWAYS_RX

 	USB_OTG_Core.rx_requested = 1;

	USBH_BulkReceiveData( &USB_OTG_Core, 
		USB_OTG_Core.rx_buffers[USB_OTG_Core.interrupt_buffer], 
		USBH_MSC_MPS_SIZE, 
		net.hc_num_in);

#else
	USB_OTG_Core.host.XferCnt[net.hc_num_in] = 0;


	USBH_BulkReceiveData( &USB_OTG_Core, 
		net.in_buffer, 
//		USBH_MSC_MPS_SIZE, 
		IN_BUFFER_SIZE,
		net.hc_num_in);

#endif

}




// Send a multi-fragment URB
// Called by rtl_usb_tx
void usb_start_transmit(unsigned char *data, int size, uint8_t hc_num)
{
	net.total_tx += size;

// Store information
	USB_OTG_Core.host.tx_ptr[hc_num] = data;
	USB_OTG_Core.host.tx_remaining[hc_num] = size;


// Send 1st fragment
	int fragment = size;
	if(fragment > 64) fragment = 64;
	if(fragment % 4) fragment += 4 - (fragment % 4);

	USBH_BulkSendData (&USB_OTG_Core,
		data, 
		fragment, 
		hc_num);
	USB_OTG_Core.host.tx_ptr[hc_num] += fragment;
	USB_OTG_Core.host.tx_remaining[hc_num] -= fragment;
}


// Add RTL header & send it
void rtl_usb_tx(unsigned char *data,
	int len,
	enum rtl_txq qnum)
{
	int ep_num = ep_mapping[qnum];
	struct rtl_priv *rtlpriv = rtl_priv(&net.hw);
	struct rtl_tx_desc *pdesc = NULL;
	struct ieee80211_hdr *hdr = (struct ieee80211_hdr *)(data);
	struct rtl_tcb_desc tcb_desc;
	struct ieee80211_tx_info info;


/*
 * TRACE
 * print_number(len);
 * print_number(total_tx);
 * total_tx++;
 */

	bzero(&info, sizeof(struct ieee80211_tx_info));
	bzero(&tcb_desc, sizeof(struct rtl_tcb_desc));

// HACK
	hdr->duration_id = DURATION_VALUE;

	hdr->seq_ctrl = net.seq_ctrl;
	net.seq_ctrl += 0x10;
	net.seq_ctrl &= IEEE80211_SCTL_SEQ;

// data needs extra room for the header
	unsigned char *new_data = data;
	rtl92cu_tx_fill_desc(&net.hw,
		hdr, 
		(u8 *)pdesc, 
		&info, 
		data, 
		&len,
		0, 
		&tcb_desc,
		qnum,
		&new_data);




	usb_start_transmit(new_data, len, net.hc_num_out[ep_num]);



//TRACE2
//print_text("len=");
//print_number(len);
//print_buffer(new_data, len);
//flush_uart();
}



void _usbctrl_vendorreq_sync_read(uint8_t request,
	uint16_t value, 
	uint16_t index, 
	uint8_t *data,
	uint16_t len)
{
// usbh_def.h: USB_Setup_TypeDef
	USB_Host.Control.setup.b.bmRequestType = REALTEK_USB_VENQT_READ;
  	USB_Host.Control.setup.b.bRequest = request;
  	USB_Host.Control.setup.b.wValue = value;
  	USB_Host.Control.setup.b.wIndex = index;
  	USB_Host.Control.setup.b.wLength = len;           

//TRACE
//print_buffer(USB_Host.Control.setup.d8, 8);

// Synchronous
	USBH_Status status;
//TRACE
	do
	{
 		status = USBH_CtlReq(&USB_OTG_Core, &USB_Host, data, len);
		handle_uart();
		handle_usb();
	} while(status == USBH_BUSY);
//TRACE

	do
	{
		handle_uart();
		handle_usb();
	} while(USB_Host.gState == HOST_CTRL_XFER);
//TRACE


/*
 * TRACE
 * print_text("addr=0x");
 * print_hex(value);
 * print_text(" ");
 * print_buffer(data, len);
 * flush_uart();
 */

}


void _usbctrl_vendorreq_async_write(uint8_t request,
	uint16_t value, 
	uint16_t index, 
	uint8_t *data,
	uint16_t len)
{
// usbh_def.h: USB_Setup_TypeDef
	USB_Host.Control.setup.b.bmRequestType = REALTEK_USB_VENQT_WRITE;
  	USB_Host.Control.setup.b.bRequest = request;
  	USB_Host.Control.setup.b.wValue = value;
  	USB_Host.Control.setup.b.wIndex = index;
  	USB_Host.Control.setup.b.wLength = len;           

/*
 * TRACE
 * print_text("addr=0x");
 * print_hex(value);
 * print_text(" ");
 * print_buffer(data, len);
 * flush_uart();
 */


// Synchronous
	USBH_Status status;
	do
	{
 		status = USBH_CtlReq(&USB_OTG_Core, &USB_Host, data, len);
		handle_uart();
		handle_usb();
	} while(status == USBH_BUSY);

	do
	{
		handle_uart();
		handle_usb();
	} while(USB_Host.gState == HOST_CTRL_XFER);



// Read it back
// usbh_def.h: USB_Setup_TypeDef
	USB_Host.Control.setup.b.bmRequestType = REALTEK_USB_VENQT_READ;
  	USB_Host.Control.setup.b.bRequest = request;
  	USB_Host.Control.setup.b.wValue = value;
  	USB_Host.Control.setup.b.wIndex = index;
  	USB_Host.Control.setup.b.wLength = len;           

//TRACE

// Synchronous
	unsigned char temp[4];
	do
	{
 		status = USBH_CtlReq(&USB_OTG_Core, &USB_Host, temp, len);
		handle_uart();
		handle_usb();
	} while(status == USBH_BUSY);
//TRACE

	do
	{
		handle_uart();
		handle_usb();
	} while(USB_Host.gState == HOST_CTRL_XFER);
//TRACE

}


void _usb_write_async(uint32_t addr, u8 *data, int size)
{
	_usbctrl_vendorreq_async_write(
		REALTEK_USB_VENQT_CMD_REQ, // request
		(uint16_t)(addr&0x0000ffff),  // value
		REALTEK_USB_VENQT_CMD_IDX, // index
		data,
		size);
}

void _usb_read_sync(uint32_t addr, unsigned char *data, int size)
{
	_usbctrl_vendorreq_sync_read(
		REALTEK_USB_VENQT_CMD_REQ, // request
		(u16)addr, // wvalue
		REALTEK_USB_VENQT_CMD_IDX, // index
		data, 
		size);
	
}


// These only work in little endian
uint32_t _usb_read32_sync(struct rtl_priv *rtlpriv, uint32_t addr)
{
	uint32_t data = 0;
	_usb_read_sync(addr, (u8*)&data, 4);
	return data;
}

uint16_t _usb_read16_sync(struct rtl_priv *rtlpriv, uint32_t addr)
{
// Data alignment problem if we only allocate a byte
	uint32_t data = 0;
	_usb_read_sync(addr, (u8*)&data, 2);
	return data & 0xffff;
}

uint8_t _usb_read8_sync(struct rtl_priv *rtlpriv, uint32_t addr)
{
// Data alignment problem if we only allocate a byte
	uint32_t data = 0;
	_usb_read_sync(addr, (u8*)&data, 1);
	return data & 0xff;
}

void _usb_write8_async(struct rtl_priv *rtlpriv, uint32_t addr, u8 value)
{
// Data alignment problem if we only allocate a byte
	uint32_t data = value;
	_usb_write_async(addr, (unsigned char*)&value, 1);
}

void _usb_write16_async(struct rtl_priv *rtlpriv, uint32_t addr, uint16_t value)
{
// Data alignment problem if we only allocate a byte
	uint32_t data = value;
	_usb_write_async(addr, (unsigned char*)&value, 2);
}


void _usb_write32_async(struct rtl_priv *rtlpriv, uint32_t addr, uint32_t value)
{
	_usb_write_async(addr, (unsigned char*)&value, 4);
}




void _rtl_usb_init_sw(struct ieee80211_hw *hw)
{
	struct rtl_mac *mac = rtl_mac(rtl_priv(hw));
	struct rtl_hal *rtlhal = rtl_hal(rtl_priv(hw));
	struct rtl_ps_ctl *ppsc = rtl_psc(rtl_priv(hw));
	struct rtl_usb *rtlusb = rtl_usbdev(rtl_usbpriv(hw));

	rtlusb->out_ep_nums = 2;
	rtlusb->out_queue_sel = 5;

	rtlhal->hw = hw;
	ppsc->inactiveps = false;
	ppsc->leisure_ps = false;
	ppsc->fwctrl_lps = false;
	ppsc->reg_fwctrl_lps = 3;
	ppsc->reg_max_lps_awakeintvl = 5;
	ppsc->fwctrl_psmode = FW_PS_DTIM_MODE;

	 /* IBSS */
	mac->beacon_interval = 100;

	 /* AMPDU */
	mac->min_space_cfg = 0;
	mac->max_mss_density = 0;

	/* set sane AMPDU defaults */
	mac->current_ampdu_density = 7;
	mac->current_ampdu_factor = 3;

	/* QOS */
	rtlusb->acm_method = eAcmWay2_SW;

	/* IRQ */
	/* HIMR - turn all on */
	rtlusb->irq_mask[0] = 0xFFFFFFFF;
	/* HIMR_EX - turn all on */
	rtlusb->irq_mask[1] = 0xFFFFFFFF;
	rtlusb->disableHWSM =  true;
}





int rtl_usb_probe( /* struct usb_interface *intf,
			const struct usb_device_id *id */ )
{
	int err;
	struct ieee80211_hw *hw = NULL;
	struct rtl_priv *rtlpriv = NULL;
	struct usb_device	*udev;
	struct rtl_usb_priv *usb_priv;



//TRACE



	hw = ieee80211_alloc_hw(sizeof(struct rtl_priv) +
				sizeof(struct rtl_usb_priv), 0 /* &rtl_ops */ );
	if (!hw) {
		TRACE
//		RT_ASSERT(false, ("%s : ieee80211 alloc failed\n", __func__));
//		return -ENOMEM;
	}
	rtlpriv = hw->priv;
//TRACE



//	SET_IEEE80211_DEV(hw, &intf->dev);
//	udev = interface_to_usbdev(intf);
//	usb_get_dev(udev);
	usb_priv = rtl_usbpriv(hw);

	
//TRACE


//TRACE
//	memset(usb_priv, 0, sizeof(*usb_priv));
//	usb_priv->dev.intf = intf;
//	usb_priv->dev.udev = udev;
//	usb_set_intfdata(intf, hw);
	/* init cfg & intf_ops */
	rtlpriv->rtlhal.interface = INTF_USB;
	
	rtlpriv->cfg = (struct rtl_hal_cfg*)&rtl92cu_hal_cfg;
//	rtlpriv->cfg = (struct rtl_hal_cfg *)(id->driver_info);
//	rtlpriv->intf_ops = &rtl_usb_ops;
//	rtl_dbgp_flag_init(hw);
	/* Init IO handler */
	rtlpriv->io.write8_async	= _usb_write8_async;
	rtlpriv->io.write16_async	= _usb_write16_async;
	rtlpriv->io.write32_async	= _usb_write32_async;
	rtlpriv->io.read8_sync		= _usb_read8_sync;
	rtlpriv->io.read16_sync		= _usb_read16_sync;
	rtlpriv->io.read32_sync		= _usb_read32_sync;
//	_rtl_usb_io_handler_init(&udev->dev, hw);


//TRACE

	rtlpriv->cfg->ops->read_chip_version(hw);
/*like read eeprom and so on */

//TRACE
	rtlpriv->cfg->ops->read_eeprom_info(hw);

//TRACE
	if (rtlpriv->cfg->ops->init_sw_vars(hw)) {
		RT_TRACE(rtlpriv, COMP_ERR, DBG_EMERG,
			 ("Can't init_sw_vars.\n"));
		goto error_out;
	}

//TRACE
	rtlpriv->cfg->ops->init_sw_leds(hw);
//	err = _rtl_usb_init(hw);
	_rtl_usb_init_sw(hw);
	/* Init mac80211 sw */
	err = rtl_init_core(hw);
//TRACE
	if (err) {
		RT_TRACE(rtlpriv, COMP_ERR, DBG_EMERG,
			 ("Can't allocate sw for mac80211.\n"));
		goto error_out;
	}

	/*init rfkill */
	/* rtl_init_rfkill(hw); */

//	err = ieee80211_register_hw(hw);
	if (err) {
		RT_TRACE(rtlpriv, COMP_INIT, DBG_EMERG,
			 ("Can't register mac80211 hw.\n"));
		goto error_out;
	} else {
		rtlpriv->mac80211.mac80211_registered = 1;
	}
//TRACE
	set_bit(RTL_STATUS_INTERFACE_START, &rtlpriv->status);
	return 0;



error_out:
//	rtl_deinit_core(hw);
//	_rtl_usb_io_handler_release(hw);
//	ieee80211_free_hw(hw);
//	usb_put_dev(udev);
//	return -ENODEV;
	return 1;
}


int rtl_usb_start(struct ieee80211_hw *hw)
{
	int err;
	struct rtl_priv *rtlpriv = rtl_priv(hw);
	struct rtl_hal *rtlhal = rtl_hal(rtl_priv(hw));
	struct rtl_usb *rtlusb = rtl_usbdev(rtl_usbpriv(hw));

	total_tx = 0;
	

// rtl8192cu/sw.c
	err = rtlpriv->cfg->ops->hw_init(hw);
	
	if(err)
	{
		TRACE
		print_text("rtl_usb_start failed\n");
	}

	rtl_init_rx_config(hw);


	/* Enable software */
	SET_USB_START(rtlusb);
	/* should after adapter start and interrupt enable. */
	set_hal_start(rtlhal);

	return err;
}


#endif // USE_WIFI








